@@ -2,13 +2,9 @@
 //  Copyright (c) 2000-2002
 //  Joerg Walter, Mathias Koch
 //
-//  Permission to use, copy, modify, distribute and sell this software
-//  and its documentation for any purpose is hereby granted without fee,
-//  provided that the above copyright notice appear in all copies and
-//  that both that copyright notice and this permission notice appear
-//  in supporting documentation.  The authors make no representations
-//  about the suitability of this software for any purpose.
-//  It is provided "as is" without express or implied warranty.
+//  Distributed under the Boost Software License, Version 1.0. (See
+//  accompanying file LICENSE_1_0.txt or copy at
+//  http://www.boost.org/LICENSE_1_0.txt)
 //
 //  The authors gratefully acknowledge the support of
 //  GeNeSys mbH & Co. KG in producing this work.
@@ -43,14 +39,14 @@ namespace boost { namespace numeric { namespace ublas {
         typedef typename boost::mpl::if_<boost::is_const<E>,
                                           typename E::const_reference,
                                           typename E::reference>::type reference;
-        typedef E refered_type;
+        typedef E referred_type;
         typedef const self_type const_closure_type;
-        typedef const_closure_type closure_type;
+        typedef self_type closure_type;
         typedef typename E::storage_category storage_category;
 
         // Construction and destruction
         BOOST_UBLAS_INLINE
-        explicit vector_reference (refered_type &e):
+        explicit vector_reference (referred_type &e):
             e_ (e) {}
 
         // Accessors
@@ -62,11 +58,11 @@ namespace boost { namespace numeric { namespace ublas {
     public:
         // Expression accessors - const correct
         BOOST_UBLAS_INLINE
-        const refered_type &expression () const {
+        const referred_type &expression () const {
             return e_;
         }
         BOOST_UBLAS_INLINE
-        refered_type &expression () {
+        referred_type &expression () {
             return e_;
         }
 
@@ -227,7 +223,7 @@ namespace boost { namespace numeric { namespace ublas {
         }
 
     private:
-        refered_type &e_;
+        referred_type &e_;
     };
 
 
@@ -385,6 +381,10 @@ namespace boost { namespace numeric { namespace ublas {
             const_reference operator * () const {
                 return functor_type::apply (*it_);
             }
+            BOOST_UBLAS_INLINE
+            const_reference operator [] (difference_type n) const {
+                return *(*this + n);
+            }
 
             // Index
             BOOST_UBLAS_INLINE
@@ -638,14 +638,23 @@ namespace boost { namespace numeric { namespace ublas {
                             const const_subiterator2_type &it2, const const_subiterator2_type &it2_end):
                 container_const_reference<self_type> (vb), i_ (i), it1_ (it1), it1_end_ (it1_end), it2_ (it2), it2_end_ (it2_end) {}
 
+        private: 
             // Dense specializations
             BOOST_UBLAS_INLINE
             void increment (dense_random_access_iterator_tag) {
-                ++ i_, ++ it1_, ++ it2_;
+                ++ i_; ++ it1_; ++ it2_;
             }
             BOOST_UBLAS_INLINE
             void decrement (dense_random_access_iterator_tag) {
-                -- i_, -- it1_, -- it2_;
+                -- i_; -- it1_; -- it2_;
+            }
+            BOOST_UBLAS_INLINE
+            void increment (dense_random_access_iterator_tag, difference_type n) {
+                i_ += n; it1_ += n; it2_ += n;
+            }
+            BOOST_UBLAS_INLINE
+            void decrement (dense_random_access_iterator_tag, difference_type n) {
+                i_ -= n; it1_ -= n; it2_ -= n;
             }
             BOOST_UBLAS_INLINE
             value_type dereference (dense_random_access_iterator_tag) const {
@@ -674,6 +683,28 @@ namespace boost { namespace numeric { namespace ublas {
                 -- i_;
             }
             BOOST_UBLAS_INLINE
+            void increment (packed_random_access_iterator_tag, difference_type n) {
+                while (n > 0) {
+                    increment (packed_random_access_iterator_tag ());
+                    --n;
+                }
+                while (n < 0) {
+                    decrement (packed_random_access_iterator_tag ());
+                    ++n;
+                }
+            }
+            BOOST_UBLAS_INLINE
+            void decrement (packed_random_access_iterator_tag, difference_type n) {
+                while (n > 0) {
+                    decrement (packed_random_access_iterator_tag ());
+                    --n;
+                }
+                while (n < 0) {
+                    increment (packed_random_access_iterator_tag ());
+                    ++n;
+                }
+            }
+            BOOST_UBLAS_INLINE
             value_type dereference (packed_random_access_iterator_tag) const {
                 value_type t1 = value_type/*zero*/();
                 if (it1_ != it1_end_)
@@ -724,6 +755,28 @@ namespace boost { namespace numeric { namespace ublas {
                 i_ = (std::max) (index1, index2);
             }
             BOOST_UBLAS_INLINE
+            void increment (sparse_bidirectional_iterator_tag, difference_type n) {
+                while (n > 0) {
+                    increment (sparse_bidirectional_iterator_tag ());
+                    --n;
+                }
+                while (n < 0) {
+                    decrement (sparse_bidirectional_iterator_tag ());
+                    ++n;
+                }
+            }
+            BOOST_UBLAS_INLINE
+            void decrement (sparse_bidirectional_iterator_tag, difference_type n) {
+                while (n > 0) {
+                    decrement (sparse_bidirectional_iterator_tag ());
+                    --n;
+                }
+                while (n < 0) {
+                    increment (sparse_bidirectional_iterator_tag ());
+                    ++n;
+                }
+            }
+            BOOST_UBLAS_INLINE
             value_type dereference (sparse_bidirectional_iterator_tag) const {
                 value_type t1 = value_type/*zero*/();
                 if (it1_ != it1_end_)
@@ -736,6 +789,7 @@ namespace boost { namespace numeric { namespace ublas {
                 return functor_type::apply (t1, t2);
             }
 
+        public: 
             // Arithmetic
             BOOST_UBLAS_INLINE
             const_iterator &operator ++ () {
@@ -749,12 +803,12 @@ namespace boost { namespace numeric { namespace ublas {
             }
             BOOST_UBLAS_INLINE
             const_iterator &operator += (difference_type n) {
-                i_ += n, it1_ += n, it2_ += n;
+                increment (iterator_category (), n);
                 return *this;
             }
             BOOST_UBLAS_INLINE
             const_iterator &operator -= (difference_type n) {
-                i_ -= n, it1_ -= n, it2_ -= n;
+                decrement (iterator_category (), n);
                 return *this;
             }
             BOOST_UBLAS_INLINE
@@ -768,6 +822,10 @@ namespace boost { namespace numeric { namespace ublas {
             const_reference operator * () const {
                 return dereference (iterator_category ());
             }
+            BOOST_UBLAS_INLINE
+            const_reference operator [] (difference_type n) const {
+                return *(*this + n);
+            }
 
             // Index
             BOOST_UBLAS_INLINE
@@ -1031,6 +1089,10 @@ namespace boost { namespace numeric { namespace ublas {
             const_reference operator * () const {
                 return functor_type::apply (it1_, *it2_);
             }
+            BOOST_UBLAS_INLINE
+            const_reference operator [] (difference_type n) const {
+                return *(*this + n);
+            }
 
             // Index
             BOOST_UBLAS_INLINE
@@ -1108,7 +1170,9 @@ namespace boost { namespace numeric { namespace ublas {
     // (t * v) [i] = t * v [i]
     template<class T1, class E2>
     BOOST_UBLAS_INLINE
+    typename enable_if< is_convertible<T1, typename E2::value_type >,    
     typename vector_binary_scalar1_traits<const T1, E2, scalar_multiplies<T1, typename E2::value_type> >::result_type
+    >::type
     operator * (const T1 &e1,
                 const vector_expression<E2> &e2) {
         typedef typename vector_binary_scalar1_traits<const T1, E2, scalar_multiplies<T1, typename E2::value_type> >::expression_type expression_type;
@@ -1252,6 +1316,10 @@ namespace boost { namespace numeric { namespace ublas {
             const_reference operator * () const {
                 return functor_type::apply (*it1_, it2_);
             }
+            BOOST_UBLAS_INLINE
+            const_reference operator [] (difference_type n) const {
+                return *(*this + n);
+            }
 
             // Index
             BOOST_UBLAS_INLINE
@@ -1329,7 +1397,9 @@ namespace boost { namespace numeric { namespace ublas {
     // (v * t) [i] = v [i] * t
     template<class E1, class T2>
     BOOST_UBLAS_INLINE
+    typename enable_if< is_convertible<T2, typename E1::value_type >,    
     typename vector_binary_scalar2_traits<E1, const T2, scalar_multiplies<typename E1::value_type, T2> >::result_type
+    >::type
     operator * (const vector_expression<E1> &e1,
                 const T2 &e2) {
         typedef typename vector_binary_scalar2_traits<E1, const T2, scalar_multiplies<typename E1::value_type, T2> >::expression_type expression_type;
@@ -1339,7 +1409,9 @@ namespace boost { namespace numeric { namespace ublas {
     // (v / t) [i] = v [i] / t
     template<class E1, class T2>
     BOOST_UBLAS_INLINE
+    typename enable_if< is_convertible<T2, typename E1::value_type >,    
     typename vector_binary_scalar2_traits<E1, const T2, scalar_divides<typename E1::value_type, T2> >::result_type
+    >::type
     operator / (const vector_expression<E1> &e1,
                 const T2 &e2) {
         typedef typename vector_binary_scalar2_traits<E1, const T2, scalar_divides<typename E1::value_type, T2> >::expression_type expression_type;
@@ -1357,9 +1429,8 @@ namespace boost { namespace numeric { namespace ublas {
         typedef typename E::const_iterator::iterator_category iterator_category;
         typedef vector_scalar_unary<E, F> self_type;
     public:
-        typedef typename F::size_type size_type;
-        typedef typename F::difference_type difference_type;
         typedef typename F::result_type value_type;
+        typedef typename E::difference_type difference_type;
         typedef const self_type const_closure_type;
         typedef const_closure_type closure_type;
         typedef unknown_storage_tag storage_category;
@@ -1431,9 +1502,9 @@ namespace boost { namespace numeric { namespace ublas {
     // sum v = sum (v [i])
     template<class E>
     BOOST_UBLAS_INLINE
-    typename vector_scalar_unary_traits<E, vector_sum<typename E::value_type> >::result_type
+    typename vector_scalar_unary_traits<E, vector_sum<E> >::result_type
     sum (const vector_expression<E> &e) {
-        typedef typename vector_scalar_unary_traits<E, vector_sum<typename E::value_type> >::expression_type expression_type;
+        typedef typename vector_scalar_unary_traits<E, vector_sum<E> >::expression_type expression_type;
         return expression_type (e ());
     }
 
@@ -1441,9 +1512,9 @@ namespace boost { namespace numeric { namespace ublas {
     // complex: norm_1 v = sum (abs (real (v [i])) + abs (imag (v [i])))
     template<class E>
     BOOST_UBLAS_INLINE
-    typename vector_scalar_unary_traits<E, vector_norm_1<typename E::value_type> >::result_type
+    typename vector_scalar_unary_traits<E, vector_norm_1<E> >::result_type
     norm_1 (const vector_expression<E> &e) {
-        typedef typename vector_scalar_unary_traits<E, vector_norm_1<typename E::value_type> >::expression_type expression_type;
+        typedef typename vector_scalar_unary_traits<E, vector_norm_1<E> >::expression_type expression_type;
         return expression_type (e ());
     }
 
@@ -1451,9 +1522,9 @@ namespace boost { namespace numeric { namespace ublas {
     // complex: norm_2 v = sqrt (sum (v [i] * conj (v [i])))
     template<class E>
     BOOST_UBLAS_INLINE
-    typename vector_scalar_unary_traits<E, vector_norm_2<typename E::value_type> >::result_type
+    typename vector_scalar_unary_traits<E, vector_norm_2<E> >::result_type
     norm_2 (const vector_expression<E> &e) {
-        typedef typename vector_scalar_unary_traits<E, vector_norm_2<typename E::value_type> >::expression_type expression_type;
+        typedef typename vector_scalar_unary_traits<E, vector_norm_2<E> >::expression_type expression_type;
         return expression_type (e ());
     }
 
@@ -1461,18 +1532,18 @@ namespace boost { namespace numeric { namespace ublas {
     // complex: norm_inf v = maximum (maximum (abs (real (v [i])), abs (imag (v [i]))))
     template<class E>
     BOOST_UBLAS_INLINE
-    typename vector_scalar_unary_traits<E, vector_norm_inf<typename E::value_type> >::result_type
+    typename vector_scalar_unary_traits<E, vector_norm_inf<E> >::result_type
     norm_inf (const vector_expression<E> &e) {
-        typedef typename vector_scalar_unary_traits<E, vector_norm_inf<typename E::value_type> >::expression_type expression_type;
+        typedef typename vector_scalar_unary_traits<E, vector_norm_inf<E> >::expression_type expression_type;
         return expression_type (e ());
     }
 
     // real: index_norm_inf v = minimum (i: abs (v [i]) == maximum (abs (v [i])))
     template<class E>
     BOOST_UBLAS_INLINE
-    typename vector_scalar_unary_traits<E, vector_index_norm_inf<typename E::value_type> >::result_type
+    typename vector_scalar_unary_traits<E, vector_index_norm_inf<E> >::result_type
     index_norm_inf (const vector_expression<E> &e) {
-        typedef typename vector_scalar_unary_traits<E, vector_index_norm_inf<typename E::value_type> >::expression_type expression_type;
+        typedef typename vector_scalar_unary_traits<E, vector_index_norm_inf<E> >::expression_type expression_type;
         return expression_type (e ());
     }
 
@@ -1490,9 +1561,8 @@ namespace boost { namespace numeric { namespace ublas {
         typedef vector_scalar_binary<E1, E2, F> self_type;
     public:
         static const unsigned complexity = 1;
-        typedef typename F::size_type size_type;
-        typedef typename F::difference_type difference_type;
         typedef typename F::result_type value_type;
+        typedef typename E1::difference_type difference_type;
         typedef const self_type const_closure_type;
         typedef const_closure_type closure_type;
         typedef unknown_storage_tag storage_category;
@@ -1523,6 +1593,7 @@ namespace boost { namespace numeric { namespace ublas {
         // Dense random access specialization
         BOOST_UBLAS_INLINE
         value_type evaluate (dense_random_access_iterator_tag) const {
+            BOOST_UBLAS_CHECK (e1_.size () == e2_.size (), external_logic());
 #ifdef BOOST_UBLAS_USE_INDEXING
             return functor_type::apply (e1_, e2_);
 #elif BOOST_UBLAS_USE_ITERATING
@@ -1540,12 +1611,14 @@ namespace boost { namespace numeric { namespace ublas {
         // Packed bidirectional specialization
         BOOST_UBLAS_INLINE
         value_type evaluate (packed_random_access_iterator_tag) const {
+            BOOST_UBLAS_CHECK (e1_.size () == e2_.size (), external_logic());
             return functor_type::apply (e1_.begin (), e1_.end (), e2_.begin (), e2_.end ());
         }
 
         // Sparse bidirectional specialization
         BOOST_UBLAS_INLINE
         value_type evaluate (sparse_bidirectional_iterator_tag) const {
+            BOOST_UBLAS_CHECK (e1_.size () == e2_.size (), external_logic());
             return functor_type::apply (e1_.begin (), e1_.end (), e2_.begin (), e2_.end (), sparse_bidirectional_iterator_tag ());
         }
 
@@ -1569,31 +1642,27 @@ namespace boost { namespace numeric { namespace ublas {
     // inner_prod (v1, v2) = sum (v1 [i] * v2 [i])
     template<class E1, class E2>
     BOOST_UBLAS_INLINE
-    typename vector_scalar_binary_traits<E1, E2, vector_inner_prod<typename E1::value_type,
-                                                                   typename E2::value_type,
+    typename vector_scalar_binary_traits<E1, E2, vector_inner_prod<E1, E2,
                                                                    typename promote_traits<typename E1::value_type,
                                                                                            typename E2::value_type>::promote_type> >::result_type
     inner_prod (const vector_expression<E1> &e1,
                 const vector_expression<E2> &e2) {
-        typedef typename vector_scalar_binary_traits<E1, E2, vector_inner_prod<typename E1::value_type,
-                                                                                           typename E2::value_type,
-                                                                                           typename promote_traits<typename E1::value_type,
-                                                                                                                               typename E2::value_type>::promote_type> >::expression_type expression_type;
+        typedef typename vector_scalar_binary_traits<E1, E2, vector_inner_prod<E1, E2,
+                                                                   typename promote_traits<typename E1::value_type,
+                                                                                           typename E2::value_type>::promote_type> >::expression_type expression_type;
         return expression_type (e1 (), e2 ());
     }
 
     template<class E1, class E2>
     BOOST_UBLAS_INLINE
-    typename vector_scalar_binary_traits<E1, E2, vector_inner_prod<typename E1::value_type,
-                                                                   typename E2::value_type,
+    typename vector_scalar_binary_traits<E1, E2, vector_inner_prod<E1, E2,
                                                                    typename type_traits<typename promote_traits<typename E1::value_type,
                                                                                                                 typename E2::value_type>::promote_type>::precision_type> >::result_type
     prec_inner_prod (const vector_expression<E1> &e1,
                      const vector_expression<E2> &e2) {
-        typedef typename vector_scalar_binary_traits<E1, E2, vector_inner_prod<typename E1::value_type,
-                                                                                           typename E2::value_type,
-                                                                                           typename type_traits<typename promote_traits<typename E1::value_type,
-                                                                                                                                                                typename E2::value_type>::promote_type>::precision_type> >::expression_type expression_type;
+        typedef typename vector_scalar_binary_traits<E1, E2, vector_inner_prod<E1, E2,
+                                                                   typename type_traits<typename promote_traits<typename E1::value_type,
+                                                                                                                typename E2::value_type>::promote_type>::precision_type> >::expression_type expression_type;
         return expression_type (e1 (), e2 ());
     }
 
