@@ -1,13 +1,13 @@
-//  (C) Copyright Gennadiy Rozental 2001-2005.
+//  (C) Copyright Gennadiy Rozental 2001-2008.
 //  Distributed under the Boost Software License, Version 1.0.
 //  (See accompanying file LICENSE_1_0.txt or copy at 
 //  http://www.boost.org/LICENSE_1_0.txt)
 
 //  See http://www.boost.org/libs/test for the library home page.
 //
-//  File        : $RCSfile: prg_exec_monitor.hpp,v $
+//  File        : $RCSfile$
 //
-//  Version     : $Revision: 1.9 $
+//  Version     : $Revision: 49312 $
 //
 //  Description : included (vs. linked ) version of Program Execution Monitor
 // ***************************************************************************
@@ -16,22 +16,10 @@
 #define BOOST_INCLUDED_PRG_EXEC_MONITOR_HPP_071894GER
 
 #include <boost/test/impl/execution_monitor.ipp>
-
+#include <boost/test/impl/debug.ipp>
 #include <boost/test/impl/cpp_main.ipp>
 
-// ***************************************************************************
-//  Revision History :
-//  
-//  $Log: prg_exec_monitor.hpp,v $
-//  Revision 1.9  2005/02/01 06:40:07  rogeeff
-//  copyright update
-//  old log entries removed
-//  minor stilistic changes
-//  depricated tools removed
-//
-//  Revision 1.8  2005/01/22 19:22:13  rogeeff
-//  implementation moved into headers section to eliminate dependency of included/minimal component on src directory
-//
-// ***************************************************************************
+#define BOOST_TEST_INCLUDED
+#include <boost/test/prg_exec_monitor.hpp>
 
 #endif // BOOST_INCLUDED_PRG_EXEC_MONITOR_HPP_071894GER
